"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const dot_object_1 = __importDefault(require("dot-object"));
const dotenv_1 = __importDefault(require("dotenv"));
const fs_1 = require("fs");
const path_1 = __importDefault(require("path"));
const util_1 = require("util");
const _1 = require(".");
const Route_1 = __importDefault(require("./Route"));
const fsReadFile = util_1.promisify(fs_1.readFile);
const fsWriteFile = util_1.promisify(fs_1.writeFile);
const fsStat = util_1.promisify(fs_1.stat);
const CONFIG_FILE = () => path_1.default.resolve(process.env.CLI_CWD || './', '.origami');
var config;
(function (config_1) {
    /**
     * Injects origami_x_y environment variables into a Origami.config and replaces
     * aliased keys in objects (eg: {'/path/to/plugin.js:myPlugin': true})
     * @returns {Object} Object of origami environment variables
    */
    config_1.env = (obj = {}) => {
        dotenv_1.default.config({
            path: path_1.default.join(process.env.CLI_CWD || process.cwd(), '.env')
        });
        // A map of aliased files to their alias
        // EG: '/path/to/plugin.js:myPlugin' => {'myplugin': '/path/to/plugin.js'}
        const aliases = {
            plugins: {}
        };
        // Replace all aliased local paths (eg: '/path/to/plugin:withAlias') to the aliases
        // Also removes the aliases from the object keys
        if (obj.plugins) {
            Object.entries(obj.plugins).forEach(([plugin, options]) => {
                // If plugin is a file not a module...
                const res = /^(\.{0,2}?\/.*):([\w-]+)$/.exec(plugin);
                if (res) {
                    obj.plugins[res[1]] = options;
                    aliases.plugins[res[2].toLowerCase()] = res[1];
                    delete obj.plugins[plugin];
                }
            });
        }
        // Inject process environment variables into config
        Object.entries(process.env)
            // Find only env variables prefixed with 'origami'
            .filter(([key, value]) => /^origami_.*$/.test(key.toLowerCase()))
            // Replace '_' with '.' and remove 'origami_'
            .map(([key, value]) => [
            key.toLowerCase()
                // Replace _ with .
                .replace(/_/g, '.')
                // Remove preceding 'origami.'
                .split('.').slice(1).join('.'),
            value
        ])
            .forEach(([key, value]) => {
            let _key = key;
            let _obj = obj;
            // If the variable is a plugin setting that is contained in aliases
            // Then update the config via the filename not the alias
            const res = /plugins\.([\w-]+)/.exec(key);
            if (res && aliases.plugins[res[1]]) {
                _key = key.split('.').slice(2).join('.');
                _obj = obj.plugins[aliases.plugins[res[1]]];
            }
            // Try set the key, however current value may be a boolean or undefined
            // meaning that deeper keys may throw error. If that's the case,
            // then set the value to an empty object, and try again
            dot_object_1.default.str(_key, value, _obj);
        });
        return obj;
    };
    /**
     * Attempt to load the .origami file at the current directory. Overwrites with any ENV variables
     * @returns {Origami.Config|Boolean} The Origami file as json, or false if it cannot be
     * found or loaded correctly
    */
    config_1.read = async (fileOrDirectory) => {
        // If there is no file provided, load the .origami file in the current dir
        let file = path_1.default.join(process.env.CLI_CWD || process.cwd(), '.origami');
        if (fileOrDirectory) {
            const stats = await fsStat(fileOrDirectory);
            if (stats.isFile())
                file = fileOrDirectory;
            // If a directory is passed, attempt to load the .origami file in the directory
            // If that's not a file return false
            if (stats.isDirectory()) {
                const joined = path_1.default.join(fileOrDirectory, '.origami');
                const fileStats = await fsStat(joined);
                if (fileStats.isFile()) {
                    process.env.CLI_CWD = fileOrDirectory;
                    process.chdir(fileOrDirectory);
                    file = path_1.default.join(process.cwd(), '.origami');
                }
                else
                    return false;
            }
        }
        let c = {};
        let fileString;
        try {
            fileString = (await fsReadFile(file)).toString();
        }
        catch (_a) {
            // No .origami file
            return false;
        }
        try {
            c = JSON.parse(fileString);
        }
        catch (e) {
            console.log('Error parsing .origami file'.red);
            throw e;
        }
        c = config_1.env(c);
        return c;
    };
    /**
     * Override/write the .origami file
     * @param file JSON config for Origami app to override
     */
    config_1.write = async (file) => {
        const TAB_SIZE = 4;
        return fsWriteFile(CONFIG_FILE(), JSON.stringify(file, null, TAB_SIZE));
    };
    config_1.validate = (config) => {
        try {
            // requireKeys([
            //     'store'
            // ], config);
        }
        catch (e) {
            return _1.error(new Error(`Origami: Missing '${e.key}' setting`));
        }
        // ------------------------------------------------------ Validate store
        // const store = `origami-store-${config.store.type}`;
        // try {
        //     require(path.resolve(process.cwd(), 'node_modules', store, 'package.json'));
        // } catch (e) {
        //     if (e.name === 'Error') {
        //         console.log(path.resolve(process.cwd(), 'node_modules', store, 'package.json'));
        //         return error(
        //                 new Error(
        //                     `Origami: Could not find store plugin '${store}'. Try running 'yarn install ${store}'`
        //                 )
        //             );
        //     }
        //     return error(e);
        // }
    };
    // Setup the plugins for the server
    config_1.setupPlugins = (config, server, context = process.cwd()) => Object.entries(config.plugins).map(([name, settings]) => server.plugin(name, settings, context));
    // Setup the apps for the server
    config_1.setupApps = (config, server) => Object.entries(config.apps).map(([name, settings]) => server.application(name, settings));
    // Setup the resources for the server API
    config_1.setupResources = (config, server, context = process.cwd()) => Object.entries(config.resources).map(([name, r]) => {
        let mPath;
        let model;
        // r is a string to the model
        if (typeof r === 'string') {
            mPath = path_1.default.resolve(context, r);
            try {
                model = require(mPath);
            }
            catch (e) {
                return _1.error(new Error(`Could not load resource ${name} at ${mPath}`));
            }
            const auth = true;
            return server.resource(name, { model, auth });
            // r is a config object
        }
        else if (r instanceof Object) {
            mPath = path_1.default.resolve(context, r.model);
            try {
                model = require(mPath);
            }
            catch (e) {
                return _1.error(new Error(`Could not load resource ${name} at ${mPath}`));
            }
            const auth = r.auth;
            return server.resource(name, { model, auth });
        }
    });
    // Setup the controllers for the server API
    config_1.setupControllers = (config, server, context = process.cwd()) => Object.entries(config.controllers).map(async ([_path, c]) => {
        let config = {
            prefix: ''
        };
        if (typeof c === 'string') {
            config.prefix = c;
        }
        else if (c instanceof Object) {
            config = Object.assign({}, config, c);
        }
        const route = new Route_1.default(config.prefix);
        await route.include(path_1.default.resolve(context, _path), config.prefix, true);
        server.useRouter(route);
    });
    // Load a directory of routes, models, config files, etc, and automatically
    // add everything into the server (EG: calling useRouter(), resource(), etc)
    const loadDirectory = (dir) => {
        // TODO:
        _1.error('Not implemented');
    };
})(config = exports.config || (exports.config = {}));
